;Fractalmorphs is a model that illustrates the principle of evolution by artificial selsection. 
; It is inspired by the Biomorphs of Richard Dawkins', "Blind Watchmaker". 
globals [   
        mutation-rate  
        fittest   
        ]

breed [ seeds ]
breed [ branches ]
breed [ squares ]

turtles-own [
        level
        pen-factor
        edge
        edge-factor
        left-angle
        right-angle
        langle-factor
        rangle-factor  ]        
seeds-own [ genome  ]
branches-own [ right? ]
        

    
to setup
     ca
     set-default-shape squares "square"
; The mutation-rate is a list that determines by how much each gene mutates in each step. 
; The gene sequence is level pen-factor edge edge-factor left-angle right-angle langle-factor rangle-factor    
     
     set mutation-rate [1 0.05 10 0.2  5 5 0.1 0.1] 
     
; Now assign the initial "genetic code" to nine "seed" turtles. The genetic code consists of 
; all the variables associated with creating different two different types of fractalmorphs (branching
; sierpinski gasket like fractals). The seeds are then mutated, planted around the screen and then grown
; according to the type of fractal chosen on the interface.

     create-seeds 9 [ 
         set heading 0
         set color red
         set level 2 + random 5
         set pen-factor 1.0
         set edge 10 + random 10
         set edge-factor 0.6 + random-float 0.7
         set left-angle random 360
         set right-angle random 360
         set langle-factor 0.5 + random-float 3
         set rangle-factor 0.5 + random-float 3
         set genome (list level pen-factor edge edge-factor left-angle right-angle langle-factor rangle-factor) ]  
     mutate
     assign-genes
     plant-seeds
     if morph-type = "Branch" [branch]
     if morph-type = "Gasket" [gasket]                  
end

; When this procedure is running it is possible to select one of the Fractalmorphs with the mouse. The
; genome of the "fittest" Fractalmorph is assigned to all the seeds, then the seeds are mutated as in the 
; setup procedure.
to go
    if mouse-down? [ 
            set fittest min-one-of seeds [distancexy mouse-xcor mouse-ycor]
            ask seeds [set genome [genome] of fittest 
            set pen-size [pen-size] of fittest]
            clear-drawing
            mutate
            assign-genes
            if morph-type = "Branch" [branch]
            if morph-type = "Gasket" [gasket]
            wait 0.1 ]
end

; Place "seed" turtles evenly around the screen, with turtle 0 at the center.
to plant-seeds
  ask turtle 0 [  setxy  ( 2 * max-pxcor / (- 3)) (2 * max-pycor / 3) ]
  ask turtle 1 [  setxy  0 (2 * max-pycor / 3) ]
  ask turtle 2 [  setxy  ( 2 * max-pxcor / 3) (2 * max-pycor / 3) ]
  ask turtle 3 [  setxy  ( 2 * max-pxcor / (- 3)) 0 ]
  ask turtle 4 [  setxy  ( 2 * max-pxcor / 3) 0 ]
  ask turtle 5 [  setxy  ( 2 * max-pxcor / (- 3)) (2 * max-pycor / (- 3)) ]
  ask turtle 6 [  setxy  0 (2 * max-pycor / (- 3)) ]
  ask turtle 7 [  setxy  ( 2 * max-pxcor / 3) (2 * max-pycor / (- 3)) ]
end

; Each seed, with the exception of the turtle at the center of the screen (the 9th seed), 
; has the gene gene corresponding to ts who number in the genome list mutated by adding
; or subracting the value of the mutation rate from the corresponding gene in the genome. 
; (Note: We only subtract if the result is a postive value for the gene.)
to mutate
  ask seeds with [who < 8 ][
      let i who
      ifelse (random 2) = 1
      [ set  genome replace-item i genome ( (item i genome) + (item i mutation-rate) )]
      [ if (item i genome) > (item i mutation-rate)
              [  set  genome replace-item i genome ( (item i genome) - (item i mutation-rate) ) ] ] ]
end

; Update the genes after mutation
to assign-genes
    ask seeds [    
      set level item 0 genome
      set pen-factor item 1 genome
      set edge item 2 genome
      set edge-factor item 3 genome
      set left-angle item 4 genome
      set right-angle item 5 genome
      set langle-factor item 6 genome
      set rangle-factor item 7 genome   ]
end    

; These are predetermined genetic codes where the Fractal-Morphs have a particular shape. 
to choose-creature
   set morph-type "Branch"
   ask seeds [
       if creature = "spider" [set genome [6 1.1 20 0.8 40 52 2.18 1.93] ]
       if creature = "fly" [ set genome  [6 1.1 10 0.9 37 49 0.9 2.1] ] 
       if creature = "flowers" [ set genome [8 0.9 20 0.8 46 44 2.49 2.51]  ]
       if creature = "moth" [ set genome [6 1 30 0.8 108 36 1.2 0.9] ]]
       clear-drawing
       mutate
       assign-genes
       branch
end

; This is the main branching procedure. It is based on a simple dichotomous tree. Each seed produces 
; two branches at a particular angle. Each of these produces branches in turn, repeating the process 
; a number of times determined by a gene called "level". The length of the branches (edge) changes
; at each level of branching by a particular factor (edge-factor). The left and right branching angles
; also change at each level by factors langle-factor and rangle-factor. To preserve bilateral symmetry 
; the main right branch is a mirror image of the main left branch. The pen-size variable is changed
; by a pen-factor gene.  Color is not changed by a separate gene, but is instead linked to the angle genes.
; Two important points. After producing new branches, old branches must die, otherwise they will 
; keep branching and grow out of control. Also, without-interruption is crucial here. Without it seeds would
; be asking the branches of other seeds to branch.

to branch 
 without-interruption [
     ask seeds [
         pendown
         hatch-branches 1 [rt right-angle
                           set right? true]
         hatch-branches 1 [lt right-angle 
                           set right? false]
         let n  level
         repeat level [ set n n - 1       
                        ask branches [
                               set edge edge * edge-factor 
                               set right-angle right-angle * rangle-factor
                               set left-angle left-angle * langle-factor
                               set pen-size pen-size * pen-factor
                               fd edge                         
                               ifelse right? [
                               hatch 1 [rt right-angle           
                                        set color right-angle * n]
                               hatch 1 [lt left-angle
                                        set color left-angle * n] 
                                        ]
                                         [
                               hatch 1 [lt right-angle           
                                        set color right-angle * n]
                               hatch 1 [rt left-angle
                                        set color left-angle * n] 
                                       ]  
                               die
                         ]
                     ]
           ask branches [die]  
        ]
      ]         
end

; This procedure is based on the Sierpinski Gasket type of fractal. The Sierpinski Gasket is
; created by repeated iteration of the following step: Given a square, replace it with three  
; squares that are reduced in size by a factor of a half and arranged in a trianglular shape 
; with two on the base and one on the top. When this step is repeated with the smaller squares
; the Sierpinski gastket is created. Different fractals can be created by rotating the squares
; by particular angles. In this procedure the elements (squares) of the fractals "branches" so 
; that the same procedures can be used, but really they do not branch. To create the fractal,
; the seed produces a new square which then is reduced in size by a factor. It then makes two 
; copies of itself, one which moves down, turns left moves forward a step and then to the right.
; The other does the mirror images of this manouever. The original square then moves forward a step.
; This then creates the desired triangular arrangment. 
; The process is repeat a number of times determined by the gene "level".
; The steps, rotating angles and the scale factors by which they changed are determined by genes.
; Color is linked to the angle gene, but is not a gene itself.

to gasket  
      clear-drawing
      ask seeds [   
          pen-up
          set size edge * 5
          set hidden? true ]
      without-interruption [
          ask seeds [
                hatch-squares 1 []
                let n level - 2
                repeat (level - 1) [ 
                      ask squares [
                          set size size * (edge-factor * 0.5)
                          set right-angle right-angle * rangle-factor
                          set left-angle left-angle * langle-factor
                          let step size * (edge-factor * 0.5)
                          let angle pen-factor * 90
                          hatch 1 [ 
                              bk step lt left-angle fd step rt right-angle  
                              set color  1.2 * left-angle * n
                              rt angle  ]
                          hatch 1 [ bk step rt left-angle fd step lt right-angle  
                              set color  1.2 * left-angle * n 
                              lt angle]  
                          fd step   ]  ]  
                      ask squares [stamp die]  
                      set n n - 1]  ]
                     
end

;;  ***  Copyright notice  ***
;;
;;   Copyright 2006 David McAvity
;;
;; This model was created at the Evergeen State College, in Olympia Washington
;; as part of a series of applets to illustrate principles in physics and biology. Funding was provided
;; the Plato Royalty Grant
;; 
;; The model may be freely used, modified and redistribued provided this copyright is included and
;; it not used for profit.
;;
;; Contact David McAvity at mcavityd@evergreen.edu if you have questions about its use.
 
@#$#@#$#@
GRAPHICS-WINDOW
191
13
702
545
250
250
1.0
1
10
1
1
1
0
1
1
1
-250
250
-250
250
0
0
1
ticks

BUTTON
2
22
93
70
NIL
setup\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
95
22
177
70
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

MONITOR
192
549
703
594
Genome of Fittest
[genome] of turtle 0
2
1
11

SLIDER
10
253
182
286
start-level
start-level
1
8
4
1
1
NIL
HORIZONTAL

SLIDER
10
289
182
322
start-edge
start-edge
1
20
20
1
1
NIL
HORIZONTAL

SLIDER
10
325
182
358
start-right-angle
start-right-angle
0
180
173
1
1
NIL
HORIZONTAL

SLIDER
11
363
183
396
start-left-angle
start-left-angle
0
180
41
1
1
NIL
HORIZONTAL

CHOOSER
10
204
182
249
Morph-Type
Morph-Type
"Branch" "Gasket"
0

CHOOSER
10
434
183
479
creature
creature
"spider" "moth" "fly" "flowers"
1

BUTTON
9
399
183
432
NIL
choose-creature
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

TEXTBOX
11
79
181
190
----------------------------------------\nPress setup, then go. Select your prefered Fractalmorph with your mouse. Your selection will appear in the center, with mutated morphs surrounding it.\n-----------------------------------------
11
0.0
0

@#$#@#$#@

WHAT IS IT?
-----------
Fractalmorph is a NetLogo model that illustrates the principle of evolution by artificial selection. It is inspired by the Biomorphs of Richard Dawkins', "The Blind Watchmaker". 

HOW IT WORKS
------------
After setup, you are presented with nine simple figures, based either on a branching structure, or fractal structure related to the Sierpinski gasket. These are the Fractalmorphs whose structure is determined by eight different variables or "genes". Each Fractalmorph differs from the central one by a single mutation in one of the genes. The user selects their favorite Fractalmorph and this becomes the new central morph in the next generation, with slightly mutated children surrounding it. As the user continues to select morphs, the shapes gradually change to new forms.


HOW TO USE IT
-------------
Choose a morph type (either branch or gasket), then click setup and then go. Keep selecting your favorite morph. The morphs are initially very simple, but as you continue selecting you will see that they can quickly become more complex. You have some scope to preselect the starting genes of your morphs also, although since the space of all morphs is huge, it is more satisfying exploring this space through evolution by artificial selection then by randomly picking values for genes.


THINGS TO NOTICE
----------------
After playing around for a while you should begin to notice how the different genes affect the morphs. For example, the top left morph is the one that has the "level" gene mutated. A higher value for this gene results in a more complex morph. By continuously selecting morphs in a particular location it is possible to mutate just one gene and see how it affects the form of the morphs.


THINGS TO TRY
-------------
If you don't like any of your choices select the central morph again. This has the effect of mutating the genes again. Some of the morphs will then look different. Also, try out the preselected morphs to see some interesting shapes. Try switching between the two different morph types. The gene names are the same, but they are used differently by the different procedures. What looks visually appealing for one type of morph may not look as good for the other type.


EXTENDING THE MODEL
-------------------
It is possible to create more complicated morphs by adding more genes to the existing morph types. You would then need either create more seeds to show all the mutations, or keep the eight mutated morphs and randomly select which genes to morph. You might also invent a different morph type based on some other kind of fractal structure.


NETLOGO FEATURES
----------------
This model was created as a way to illustrate the use of lists in NetLogo. It also demonstrates one way to use multiple turtles to create fractals.

RELATED MODELS
--------------
See other Evolution based models in this series


CREDITS AND REFERENCES
----------------------
   Copyright 2006 David McAvity

This model was created at the Evergreen State College, in Olympia Washington
as part of a series of applets to illustrate principles in physics and biology. 

Funding was provided by the Plato Royalty Grant.
 
The model may be freely used, modified and redistributed provided this copyright is included and the resulting models are not used for profit.

Contact David McAvity at mcavityd@evergreen.edu if you have questions about its use.


@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 30 30 240

circle 2
false
0
Circle -7500403 true true 16 16 270
Circle -16777216 true false 46 46 210

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

link
true
0
Line -7500403 true 150 0 150 300

link direction
true
0
Line -7500403 true 150 150 30 225
Line -7500403 true 150 150 270 225

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
true
0
Rectangle -7500403 true true 0 0 300 300

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 60 270 150 0 240 270 15 105 285 105
Polygon -7500403 true true 75 120 105 210 195 210 225 120 150 75

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
